package com.persagy.dmp.rwd.edit.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.rwd.edit.config.web.UserUtils;
import com.persagy.dmp.rwd.edit.entity.AdminPermission;
import com.persagy.dmp.rwd.edit.entity.AdminRole;
import com.persagy.dmp.rwd.edit.entity.QAdminPermission;
import com.persagy.dmp.rwd.edit.entity.QAdminRole;
import com.persagy.dmp.rwd.edit.repository.AdminPermissionRepository;
import com.persagy.dmp.rwd.edit.repository.AdminRoleRepository;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Service
public class AdminRoleService {

    @Autowired
    private AdminRoleRepository adminRoleRepository;

    @Autowired
    private AdminPermissionRepository permissionRepository;

    @Autowired
    private CriteriaUtils criteriaUtils;

    private List<BooleanExpression> parse(ObjectNode criteria) {
        // TODO
        return null;
    }

    public PagedResponse<AdminRole> query(JacksonCriteria criteria) {
        PagedResponse<AdminRole> query = criteriaUtils.query(QAdminRole.adminRole, this::parse, criteria);

        // permissions
        PagedResponse<AdminRole> resp = new PagedResponse<>();
        resp.setCount(query.getCount());
        List<AdminRole> data = query.getData();
        if (data != null) {
            boolean withPermissions = criteria.withColumn("permissions");
            List<AdminRole> list = new ArrayList<>(data.size());
            for (AdminRole role : data) {
                AdminRole model = new AdminRole();
                BeanUtils.copyProperties(role, model, "permissions");
                if (withPermissions) {
                    model.setPermissions(role.getPermissions());
                }
                list.add(model);
            }
            resp.setData(list);
        }
        return resp;
    }

    @Transactional
    public MapResponse create(AdminRole param) {
        MapResponse response = new MapResponse();
        // TODO
        String name = param.getName();
        if (name == null) {
            response.setFail("name is required");
            return response;
        }

        long count = adminRoleRepository.count(QAdminRole.adminRole.name.eq(name));
        if (count > 0) {
            response.setFail("name is exists");
            return response;
        }

        AdminRole role = new AdminRole();
        role.setName(param.getName());
        role.setRemark(param.getRemark());
        role.setCreateUser(UserUtils.currentUserId());
        role.setStatus(1);
        adminRoleRepository.save(role);
        response.add("id", role.getId());
        return response;
    }

    @Transactional
    public MapResponse update(AdminRole param) {
        MapResponse response = new MapResponse();
        // TODO
        Integer roleId = param.getId();
        Optional<AdminRole> opt = adminRoleRepository.findById(roleId);
        if (!opt.isPresent()) {
            response.setFail("data not exists");
            return response;
        }

        AdminRole role = opt.get();
        if (param.getRemark() != null) {
            role.setRemark(param.getRemark());
        }

        // TODO
        List<AdminPermission> permissions = param.getPermissions();
        if (permissions != null) {
            role.getPermissions().clear();
            for (AdminPermission permission : permissions) {
                if (permission.getSign() == null) {
                    continue;
                }
                AdminPermission entity;
                Optional<AdminPermission> optp = permissionRepository.findOne(QAdminPermission.adminPermission.sign.eq(permission.getSign()));
                if (!optp.isPresent()) {
                    entity = new AdminPermission();
                    entity.setSign(permission.getSign());
                    entity.setStatus(1);
                    permissionRepository.save(entity);
                } else {
                    entity = optp.get();
                    if (entity.getStatus() != 1) {
                        continue;
                    }
                }
                role.getPermissions().add(entity);
            }
        }
        role.setUpdateUser(UserUtils.currentUserId());
        adminRoleRepository.save(role);
        return response;
    }

    @Transactional
    public MapResponse delete(AdminRole param) {
        MapResponse response = new MapResponse();
        // TODO
        Integer userId = param.getId();
        Optional<AdminRole> opt = adminRoleRepository.findById(userId);
        if (opt.isPresent()) {
            AdminRole role = opt.get();
            role.setStatus(0);
            role.setUpdateUser(UserUtils.currentUserId());
            adminRoleRepository.save(role);
        }
        return response;
    }
}

