package com.persagy.dmp.rwd.edit.service;

import com.persagy.common.criteria.CriteriaExpressionParser;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.ListResponse;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.rwd.edit.entity.ClassDef;
import com.persagy.dmp.rwd.edit.entity.ClassDefChangeRecord;
import com.persagy.dmp.rwd.edit.entity.QClassDefChangeRecord;
import com.persagy.dmp.rwd.edit.enumeration.EnumVersionState;
import com.persagy.dmp.rwd.edit.repository.ClassDefChangeRecordRepository;
import com.persagy.dmp.rwd.edit.repository.ClassDefRepository;
import com.persagy.dmp.rwd.enums.ObjType;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.*;

class ClassDefChangeRecordServiceTest {

    private ClassDefChangeRecordService classDefChangeRecordServiceUnderTest;

    @BeforeEach
    void setUp() {
        classDefChangeRecordServiceUnderTest = new ClassDefChangeRecordService();
        classDefChangeRecordServiceUnderTest.repository = mock(ClassDefChangeRecordRepository.class);
        classDefChangeRecordServiceUnderTest.classDefService = mock(ClassDefService.class);
        classDefChangeRecordServiceUnderTest.criteriaUtils = mock(CriteriaUtils.class);
        classDefChangeRecordServiceUnderTest.classDefRepository = mock(ClassDefRepository.class);
    }

    @Test
    void testQuery() {
        // Setup
        final JacksonCriteria criteria = JacksonCriteria.newInstance();
        when(classDefChangeRecordServiceUnderTest.criteriaUtils.query(eq(new QClassDefChangeRecord("variable")), any(CriteriaExpressionParser.class), any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());

        // Run the test
        final PagedResponse<ClassDefChangeRecord> result = classDefChangeRecordServiceUnderTest.query(criteria);

        // Verify the results
    }

    @Test
    void testCreate() {
        // Setup
        final ClassDefChangeRecord param = new ClassDefChangeRecord();
        param.setId(0);
        param.setCode("code");
        param.setGroupCode("groupCode");
        param.setObjType(ObjType.project);
        param.setName("name");
        param.setAliasCode("aliasCode");
        param.setAliasName("aliasName");
        param.setType("type");
        param.setProjectId("projectId");
        param.setMajorCode("majorCode");

        when(classDefChangeRecordServiceUnderTest.classDefService.queryClass(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());

        // Configure ClassDefChangeRecordRepository.findAll(...).
        final ClassDefChangeRecord classDefChangeRecord = new ClassDefChangeRecord();
        classDefChangeRecord.setId(0);
        classDefChangeRecord.setCode("code");
        classDefChangeRecord.setGroupCode("groupCode");
        classDefChangeRecord.setObjType(ObjType.project);
        classDefChangeRecord.setName("name");
        classDefChangeRecord.setAliasCode("aliasCode");
        classDefChangeRecord.setAliasName("aliasName");
        classDefChangeRecord.setType("type");
        classDefChangeRecord.setProjectId("projectId");
        classDefChangeRecord.setMajorCode("majorCode");
        final List<ClassDefChangeRecord> classDefChangeRecords = Arrays.asList(classDefChangeRecord);
        when(classDefChangeRecordServiceUnderTest.repository.findAll()).thenReturn(classDefChangeRecords);

        // Configure ClassDefChangeRecordRepository.save(...).
        final ClassDefChangeRecord classDefChangeRecord1 = new ClassDefChangeRecord();
        classDefChangeRecord1.setId(0);
        classDefChangeRecord1.setCode("code");
        classDefChangeRecord1.setGroupCode("groupCode");
        classDefChangeRecord1.setObjType(ObjType.project);
        classDefChangeRecord1.setName("name");
        classDefChangeRecord1.setAliasCode("aliasCode");
        classDefChangeRecord1.setAliasName("aliasName");
        classDefChangeRecord1.setType("type");
        classDefChangeRecord1.setProjectId("projectId");
        classDefChangeRecord1.setMajorCode("majorCode");
        when(classDefChangeRecordServiceUnderTest.repository.save(any(ClassDefChangeRecord.class))).thenReturn(classDefChangeRecord1);

        // Run the test
        final MapResponse result = classDefChangeRecordServiceUnderTest.create(param);

        // Verify the results
        verify(classDefChangeRecordServiceUnderTest.repository).deleteAll(Arrays.asList(new ClassDefChangeRecord()));
    }

    @Test
    void testCreate_ClassDefChangeRecordRepositoryFindAllReturnsNoItems() {
        // Setup
        final ClassDefChangeRecord param = new ClassDefChangeRecord();
        param.setId(0);
        param.setCode("code");
        param.setGroupCode("groupCode");
        param.setObjType(ObjType.project);
        param.setName("name");
        param.setAliasCode("aliasCode");
        param.setAliasName("aliasName");
        param.setType("type");
        param.setProjectId("projectId");
        param.setMajorCode("majorCode");

        when(classDefChangeRecordServiceUnderTest.classDefService.queryClass(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());
        when(classDefChangeRecordServiceUnderTest.repository.findAll()).thenReturn(Collections.emptyList());

        // Configure ClassDefChangeRecordRepository.save(...).
        final ClassDefChangeRecord classDefChangeRecord = new ClassDefChangeRecord();
        classDefChangeRecord.setId(0);
        classDefChangeRecord.setCode("code");
        classDefChangeRecord.setGroupCode("groupCode");
        classDefChangeRecord.setObjType(ObjType.project);
        classDefChangeRecord.setName("name");
        classDefChangeRecord.setAliasCode("aliasCode");
        classDefChangeRecord.setAliasName("aliasName");
        classDefChangeRecord.setType("type");
        classDefChangeRecord.setProjectId("projectId");
        classDefChangeRecord.setMajorCode("majorCode");
        when(classDefChangeRecordServiceUnderTest.repository.save(any(ClassDefChangeRecord.class))).thenReturn(classDefChangeRecord);

        // Run the test
        final MapResponse result = classDefChangeRecordServiceUnderTest.create(param);

        // Verify the results
        verify(classDefChangeRecordServiceUnderTest.repository).deleteAll(Arrays.asList(new ClassDefChangeRecord()));
    }

    @Test
    void testUpdate() {
        // Setup
        final ClassDefChangeRecord param = new ClassDefChangeRecord();
        param.setId(0);
        param.setCode("code");
        param.setGroupCode("groupCode");
        param.setObjType(ObjType.project);
        param.setName("name");
        param.setAliasCode("aliasCode");
        param.setAliasName("aliasName");
        param.setType("type");
        param.setProjectId("projectId");
        param.setMajorCode("majorCode");

        // Configure ClassDefChangeRecordRepository.findById(...).
        final ClassDefChangeRecord classDefChangeRecord1 = new ClassDefChangeRecord();
        classDefChangeRecord1.setId(0);
        classDefChangeRecord1.setCode("code");
        classDefChangeRecord1.setGroupCode("groupCode");
        classDefChangeRecord1.setObjType(ObjType.project);
        classDefChangeRecord1.setName("name");
        classDefChangeRecord1.setAliasCode("aliasCode");
        classDefChangeRecord1.setAliasName("aliasName");
        classDefChangeRecord1.setType("type");
        classDefChangeRecord1.setProjectId("projectId");
        classDefChangeRecord1.setMajorCode("majorCode");
        final Optional<ClassDefChangeRecord> classDefChangeRecord = Optional.of(classDefChangeRecord1);
        when(classDefChangeRecordServiceUnderTest.repository.findById(0)).thenReturn(classDefChangeRecord);

        // Configure ClassDefChangeRecordRepository.save(...).
        final ClassDefChangeRecord classDefChangeRecord2 = new ClassDefChangeRecord();
        classDefChangeRecord2.setId(0);
        classDefChangeRecord2.setCode("code");
        classDefChangeRecord2.setGroupCode("groupCode");
        classDefChangeRecord2.setObjType(ObjType.project);
        classDefChangeRecord2.setName("name");
        classDefChangeRecord2.setAliasCode("aliasCode");
        classDefChangeRecord2.setAliasName("aliasName");
        classDefChangeRecord2.setType("type");
        classDefChangeRecord2.setProjectId("projectId");
        classDefChangeRecord2.setMajorCode("majorCode");
        when(classDefChangeRecordServiceUnderTest.repository.save(any(ClassDefChangeRecord.class))).thenReturn(classDefChangeRecord2);

        // Run the test
        final MapResponse result = classDefChangeRecordServiceUnderTest.update(param);

        // Verify the results
    }

    @Test
    void testUpdate_ClassDefChangeRecordRepositoryFindByIdReturnsAbsent() {
        // Setup
        final ClassDefChangeRecord param = new ClassDefChangeRecord();
        param.setId(0);
        param.setCode("code");
        param.setGroupCode("groupCode");
        param.setObjType(ObjType.project);
        param.setName("name");
        param.setAliasCode("aliasCode");
        param.setAliasName("aliasName");
        param.setType("type");
        param.setProjectId("projectId");
        param.setMajorCode("majorCode");

        when(classDefChangeRecordServiceUnderTest.repository.findById(0)).thenReturn(Optional.empty());

        // Configure ClassDefChangeRecordRepository.save(...).
        final ClassDefChangeRecord classDefChangeRecord = new ClassDefChangeRecord();
        classDefChangeRecord.setId(0);
        classDefChangeRecord.setCode("code");
        classDefChangeRecord.setGroupCode("groupCode");
        classDefChangeRecord.setObjType(ObjType.project);
        classDefChangeRecord.setName("name");
        classDefChangeRecord.setAliasCode("aliasCode");
        classDefChangeRecord.setAliasName("aliasName");
        classDefChangeRecord.setType("type");
        classDefChangeRecord.setProjectId("projectId");
        classDefChangeRecord.setMajorCode("majorCode");
        when(classDefChangeRecordServiceUnderTest.repository.save(any(ClassDefChangeRecord.class))).thenReturn(classDefChangeRecord);

        // Run the test
        final MapResponse result = classDefChangeRecordServiceUnderTest.update(param);

        // Verify the results
    }

    @Test
    void testAddVersion() {
        // Setup
        when(classDefChangeRecordServiceUnderTest.repository.addVersion("version", Arrays.asList(0))).thenReturn(0);

        // Run the test
        final MapResponse result = classDefChangeRecordServiceUnderTest.addVersion("version", Arrays.asList(0));

        // Verify the results
    }

    @Test
    void testDelete() {
        // Setup
        final ClassDefChangeRecord param = new ClassDefChangeRecord();
        param.setId(0);
        param.setCode("code");
        param.setGroupCode("groupCode");
        param.setObjType(ObjType.project);
        param.setName("name");
        param.setAliasCode("aliasCode");
        param.setAliasName("aliasName");
        param.setType("type");
        param.setProjectId("projectId");
        param.setMajorCode("majorCode");

        // Configure ClassDefChangeRecordRepository.findById(...).
        final ClassDefChangeRecord classDefChangeRecord1 = new ClassDefChangeRecord();
        classDefChangeRecord1.setId(0);
        classDefChangeRecord1.setCode("code");
        classDefChangeRecord1.setGroupCode("groupCode");
        classDefChangeRecord1.setObjType(ObjType.project);
        classDefChangeRecord1.setName("name");
        classDefChangeRecord1.setAliasCode("aliasCode");
        classDefChangeRecord1.setAliasName("aliasName");
        classDefChangeRecord1.setType("type");
        classDefChangeRecord1.setProjectId("projectId");
        classDefChangeRecord1.setMajorCode("majorCode");
        final Optional<ClassDefChangeRecord> classDefChangeRecord = Optional.of(classDefChangeRecord1);
        when(classDefChangeRecordServiceUnderTest.repository.findById(0)).thenReturn(classDefChangeRecord);

        // Run the test
        final MapResponse result = classDefChangeRecordServiceUnderTest.delete(param);

        // Verify the results
        verify(classDefChangeRecordServiceUnderTest.repository).delete(any(ClassDefChangeRecord.class));
    }

    @Test
    void testDelete_ClassDefChangeRecordRepositoryFindByIdReturnsAbsent() {
        // Setup
        final ClassDefChangeRecord param = new ClassDefChangeRecord();
        param.setId(0);
        param.setCode("code");
        param.setGroupCode("groupCode");
        param.setObjType(ObjType.project);
        param.setName("name");
        param.setAliasCode("aliasCode");
        param.setAliasName("aliasName");
        param.setType("type");
        param.setProjectId("projectId");
        param.setMajorCode("majorCode");

        when(classDefChangeRecordServiceUnderTest.repository.findById(0)).thenReturn(Optional.empty());

        // Run the test
        final MapResponse result = classDefChangeRecordServiceUnderTest.delete(param);

        // Verify the results
        verify(classDefChangeRecordServiceUnderTest.repository).delete(any(ClassDefChangeRecord.class));
    }

    @Test
    void testGetSqlList() {
        // Setup
        when(classDefChangeRecordServiceUnderTest.criteriaUtils.query(eq(new QClassDefChangeRecord("variable")), any(CriteriaExpressionParser.class), any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());
        when(classDefChangeRecordServiceUnderTest.classDefService.queryClass(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());

        // Run the test
        final List<String> result = classDefChangeRecordServiceUnderTest.getSqlList("version", false);

        // Verify the results
        assertThat(result).isEqualTo(Arrays.asList("value"));
    }

    @Test
    void testUpdateStateByVersion() {
        // Setup
        when(classDefChangeRecordServiceUnderTest.criteriaUtils.query(eq(new QClassDefChangeRecord("variable")), any(CriteriaExpressionParser.class), any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());
        when(classDefChangeRecordServiceUnderTest.classDefService.queryClass(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());

        // Configure ClassDefRepository.saveAll(...).
        final ClassDef classDef = new ClassDef();
        classDef.setId("id");
        classDef.setGroupCode("groupCode");
        classDef.setCode("code");
        classDef.setObjType(ObjType.project);
        classDef.setName("name");
        classDef.setAliasCode("aliasCode");
        classDef.setAliasName("aliasName");
        classDef.setType("type");
        classDef.setProjectId("projectId");
        classDef.setMajorCode("majorCode");
        final List<ClassDef> classDefs = Arrays.asList(classDef);
        when(classDefChangeRecordServiceUnderTest.classDefRepository.saveAll(Arrays.asList(new ClassDef()))).thenReturn(classDefs);

        // Configure ClassDefChangeRecordRepository.saveAll(...).
        final ClassDefChangeRecord classDefChangeRecord = new ClassDefChangeRecord();
        classDefChangeRecord.setId(0);
        classDefChangeRecord.setCode("code");
        classDefChangeRecord.setGroupCode("groupCode");
        classDefChangeRecord.setObjType(ObjType.project);
        classDefChangeRecord.setName("name");
        classDefChangeRecord.setAliasCode("aliasCode");
        classDefChangeRecord.setAliasName("aliasName");
        classDefChangeRecord.setType("type");
        classDefChangeRecord.setProjectId("projectId");
        classDefChangeRecord.setMajorCode("majorCode");
        final List<ClassDefChangeRecord> classDefChangeRecords = Arrays.asList(classDefChangeRecord);
        when(classDefChangeRecordServiceUnderTest.repository.saveAll(Arrays.asList(new ClassDefChangeRecord()))).thenReturn(classDefChangeRecords);

        // Run the test
        classDefChangeRecordServiceUnderTest.updateStateByVersion(EnumVersionState.CANCEL, "version");

        // Verify the results
    }

    @Test
    void testUpdateStateByVersion_ClassDefRepositoryReturnsNoItems() {
        // Setup
        when(classDefChangeRecordServiceUnderTest.criteriaUtils.query(eq(new QClassDefChangeRecord("variable")), any(CriteriaExpressionParser.class), any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());
        when(classDefChangeRecordServiceUnderTest.classDefService.queryClass(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());
        when(classDefChangeRecordServiceUnderTest.classDefRepository.saveAll(Arrays.asList(new ClassDef()))).thenReturn(Collections.emptyList());

        // Configure ClassDefChangeRecordRepository.saveAll(...).
        final ClassDefChangeRecord classDefChangeRecord = new ClassDefChangeRecord();
        classDefChangeRecord.setId(0);
        classDefChangeRecord.setCode("code");
        classDefChangeRecord.setGroupCode("groupCode");
        classDefChangeRecord.setObjType(ObjType.project);
        classDefChangeRecord.setName("name");
        classDefChangeRecord.setAliasCode("aliasCode");
        classDefChangeRecord.setAliasName("aliasName");
        classDefChangeRecord.setType("type");
        classDefChangeRecord.setProjectId("projectId");
        classDefChangeRecord.setMajorCode("majorCode");
        final List<ClassDefChangeRecord> classDefChangeRecords = Arrays.asList(classDefChangeRecord);
        when(classDefChangeRecordServiceUnderTest.repository.saveAll(Arrays.asList(new ClassDefChangeRecord()))).thenReturn(classDefChangeRecords);

        // Run the test
        classDefChangeRecordServiceUnderTest.updateStateByVersion(EnumVersionState.CANCEL, "version");

        // Verify the results
    }

    @Test
    void testUpdateStateByVersion_ClassDefChangeRecordRepositoryReturnsNoItems() {
        // Setup
        when(classDefChangeRecordServiceUnderTest.criteriaUtils.query(eq(new QClassDefChangeRecord("variable")), any(CriteriaExpressionParser.class), any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());
        when(classDefChangeRecordServiceUnderTest.classDefService.queryClass(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());

        // Configure ClassDefRepository.saveAll(...).
        final ClassDef classDef = new ClassDef();
        classDef.setId("id");
        classDef.setGroupCode("groupCode");
        classDef.setCode("code");
        classDef.setObjType(ObjType.project);
        classDef.setName("name");
        classDef.setAliasCode("aliasCode");
        classDef.setAliasName("aliasName");
        classDef.setType("type");
        classDef.setProjectId("projectId");
        classDef.setMajorCode("majorCode");
        final List<ClassDef> classDefs = Arrays.asList(classDef);
        when(classDefChangeRecordServiceUnderTest.classDefRepository.saveAll(Arrays.asList(new ClassDef()))).thenReturn(classDefs);

        when(classDefChangeRecordServiceUnderTest.repository.saveAll(Arrays.asList(new ClassDefChangeRecord()))).thenReturn(Collections.emptyList());

        // Run the test
        classDefChangeRecordServiceUnderTest.updateStateByVersion(EnumVersionState.CANCEL, "version");

        // Verify the results
    }
}
